#!/usr/bin/perl -w
# Autor: Daniel Mohr
# Datum: 29.01.2007
# Version: 1.3
# Lizenz: GNU-GPL
# Dieses Programm liest /proc/net/dev aus und misst dadurch den Traffic auf einem Interface

use IO::Handle;

# Variablen als lokal definieren
my $i=0;
my $lesen="";
my $received=0;
my $transmited=0;
my $receivedalt=0;
my $transmitedalt=0;
my $trafficreceived=0;
my $traffictransmited=0;
my $trafficreceivedeinheit="Bytes/sec";
my $traffictransmitedeinheit="Bytes/sec";
my $sec = 0;
my $min = 0;
my $hour = 0;
my $mday = 0;
my $mon = 0;
my $year = 0;
my $wday = 0;
my $yday = 0;
my $isdst = 0;
my $zeitmarke = "";

# Voreinstellungen setzen und Variablen als lokal definieren:
my $interface="eth0";
my $intervall=10;
my $n=0; # Anzahl Messungen; 0 fr endlos
my $nn=10;
my $einheit=1; # dynamisch in Bits
my $art=0; # Ausgabeart, ohne Zeit

# Parameter auswerten:
if (@ARGV > 0)
  {
    for($i=0; $i<@ARGV; $i++)
      {
	if (($i < (@ARGV-1)) && ($ARGV[$i] eq "-d"))
	  {
	    $i++;
	    $intervall = $ARGV[$i];
	    if ($intervall<1)
	      {
		$intervall=10;
	      }
	  } elsif (($i < (@ARGV-1)) && ($ARGV[$i] eq "-i")) {
	    $i++;
	    $interface = $ARGV[$i];
	  } elsif (($i < (@ARGV-1)) && ($ARGV[$i] eq "-n")) {
	    $i++;
	    $n = $ARGV[$i];
	  } elsif (($i < (@ARGV-1)) && ($ARGV[$i] eq "-e")) {
	    $i++;
	    $einheit = $ARGV[$i];
	  } elsif (($i < (@ARGV-1)) && ($ARGV[$i] eq "-a")) {
	    $i++;
	    $art = $ARGV[$i];
	  } else {
	  }
      }
  } else {
    print "procnettop.pl V 1.3 \t Autor: Daniel Mohr \t Datum: 29.01.2007\nLizenz: GNU-GPL
\n";
    print "\n";
    print "Diese Programm dient dazu die bertragungsrate auf einem Netzwerkinterface\nzu ermitteln. Diese Hilfe wird nur Ausgegeben, wenn keine Parameter angegeben\nwerden. Ohne Parameter wird 10 Sekunden lang gemessen und ein Programmabbruch\nist nur durch Crtl-C mglich.\n\n";
    print "Parameter:\n";
    print "-i <interface> \t setzt das Interface, Vorgabe: eth0\n";
    print "-d <sekunden> \t setzt die Zeit pro Messung in Sekunden, Vorgabe: 10\n";
    print "-n <anzahl> \t setzt die Anzahl der Messungen, Vorgabe: 0\n";
    print " \t\t 0 entspricht unendlich.\n";
    print "-e <einheitart>\t setzt die zu verwendende Einheit, Vorgabe: 1\n";
    print " \t\t 0 Bytes, Kilobytes, Megabytes oder Gigabytes pro Sekunde\n";
    print " \t\t 1 Bits, Kilobits, Megabits oder Gigabits pro Sekunde\n";
    print " \t\t 2 Bytes pro Sekunde\n";
    print " \t\t 3 Bits pro Sekunde\n";
    print "-a <ausgabeart>\t setzt die Ausgabe, Vorgabe: 0\n";
    print " \t\t 0 ohne Zeit\n";
    print " \t\t 1 mit Zeit als Unix-Time-Stamp\n";
    print " \t\t 2 mit Datum in der Form 2007-01-29\n";
    print " \t\t 3 mit Zeit in der Form 21:23:28\n";
    print " \t\t 4 mit Datum und Zeit in der Form 2007-01-29 21:23:28\n";
    print "\n";
    print "Beispiel: procnettop.pl -i eth0 -d 60 -n 10 -e 3 -a 1\n\n";
    print "Beispiel in der bash: a=\$(tempfile);b=\$(tempfile);procnettop.pl -i eth0 -d 1 -n 10 -e 3 -a 1 > \$a;echo -e \"set terminal pdf\\nset xlabel \\\"Messungen (jede Sekunde)\\\"\\nset grid\\nset ylabel \\\"Traffic (Bits/sec)\\\"\\nset output \\\"\$b\\\"\\nset xdata time\\nset format x \\\"%H:%M:%S\\\"\\nset timefmt \\\"%s\\\"\\nplot \\\"\$a\\\" using 1:3 title \\\"Downstream\\\" with lines, \\\"\$a\\\" using 1:6 title \\\"Upstream\\\" with lines\"|gnuplot;xpdf \$b;rm \$a \$b\n\n";
    print "1 Kilobyte = 1024 Byte; 1 Megabyte = 1024 Kilobyte; 1 Gigabyte = 1024 Megabyte\n";
    print "1 Kilobit = 1024 Bit; 1 Megabit = 1024 Kilobit; 1 Gigabit = 1024 Megabit\n";
    print "down: Downstream \t up: Upstream\n";
    print "\n";
  }

print "# Interface: ".$interface." \t Messdauer: ".$intervall." \t ";
if ($n==0)
  {
    $nn=10;
    print "Anzahl Messungen: unendlich\n"
  } else {
    $nn=$n;
    print "Anzahl Messungen: ".$n."\n"
  }
for($i=0; $i<$nn; $i++)
  {
    $lesen = open(FILE,"/proc/net/dev");
    if(not defined($lesen))
      {
	die "Fehler beim ffnen der Datei '/proc/net/dev': $!\n";
      }
    while(defined($zeile = <FILE>))
      {
	if($zeile  =~ /$interface:[\s]*([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)/)
	  {
	    $received=$1;
	    $transmited=$9;
	  }
      }
    close(FILE);
    sleep($intervall);
    $receivedalt=$received;
    $transmitedalt=$transmited;
    $lesen = open(FILE,"/proc/net/dev");
    if(not defined($lesen))
      {
	die "Fehler beim ffnen der Datei '/proc/net/dev': $!\n";
      }
    while(defined($zeile = <FILE>))
      {
	if($zeile  =~ /$interface:[\s]*([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)[\s]+([\d]+)/)
	  {
	    $received=$1;
	    $transmited=$9;
	  }
      }
    close(FILE);
    $trafficreceived=($received-$receivedalt)/$intervall;
    $trafficreceivedeinheit="Bytes/sec";
    $traffictransmited=($transmited-$transmitedalt)/$intervall;
    $traffictransmitedeinheit="Bytes/sec";
    # Einheit anpassen
    if (($einheit==0) || ($einheit==2))
      {
	if ($einheit==0)
	  {
	    if ($trafficreceived>1024)
	      {
		$trafficreceived=$trafficreceived/1024;
		$trafficreceivedeinheit="Kilobytes/sec";
		if ($trafficreceived>1024)
		  {
		    $trafficreceived=$trafficreceived/1024;
		    $trafficreceivedeinheit="Megabytes/sec";
		    if ($trafficreceived>1024)
		      {
			$trafficreceived=$trafficreceived/1024;
			$trafficreceivedeinheit="Gigabytes/sec";
		      }
		  }
	      }
	    if ($traffictransmited>1024)
	      {
		$traffictransmited=$traffictransmited/1024;
		$traffictransmitedeinheit="Kilobytes/sec";
		if ($traffictransmited>1024)
		  {
		    $traffictransmited=$traffictransmited/1024;
		    $traffictransmitedeinheit="Megabytes/sec";
		    if ($traffictransmited>1024)
		      {
			$traffictransmited=$traffictransmited/1024;
			$traffictransmitedeinheit="Gigabytes/sec";
		      }
		  }
	      }
	  }
      } elsif (($einheit==1) || ($einheit==3)) {
	$trafficreceived=$trafficreceived*8;
	$trafficreceivedeinheit="Bits/sec";
	$traffictransmited=$traffictransmited*8;
	$traffictransmitedeinheit="Bits/sec";
	if ($einheit==1)
	  {
	    if ($trafficreceived>1024)
	      {
		$trafficreceived=$trafficreceived/1024;
		$trafficreceivedeinheit="Kilobits/sec";
		if ($trafficreceived>1024)
		  {
		    $trafficreceived=$trafficreceived/1024;
		    $trafficreceivedeinheit="Megabits/sec";
		    if ($trafficreceived>1024)
		      {
			$trafficreceived=$trafficreceived/1024;
			$trafficreceivedeinheit="Gigabits/sec";
		      }
		  }
	      }
	    if ($traffictransmited>1024)
	      {
		$traffictransmited=$traffictransmited/1024;
		$traffictransmitedeinheit="Kilobits/sec";
		if ($traffictransmited>1024)
		  {
		    $traffictransmited=$traffictransmited/1024;
		    $traffictransmitedeinheit="Megabits/sec";
		    if ($traffictransmited>1024)
		      {
			$traffictransmited=$traffictransmited/1024;
			$traffictransmitedeinheit="Gigabits/sec";
		      }
		  }
	      }
	  }
      }
    $trafficreceived = sprintf ("%.2f", $trafficreceived);
    $traffictransmited = sprintf ("%.2f", $traffictransmited);
    # Ausgabe
    $zeitmarke = "";
    if ($art==1)
      {
	$zeitmarke=time()." \t ";
      }
    elsif (($art==2) || ($art==3) || ($art==4))
      {
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time()); $year += 1900; $mon += 1;
	if ($art==2)
	  {
	    $zeitmarke = sprintf("%04d",$year)."-".sprintf("%02d",$mon)."-".sprintf("%02d",$mday)." \t ";
	  }
	elsif ($art==3)
	  {
	    $zeitmarke = sprintf("%02d",$hour).":".sprintf("%02d",$min).":".sprintf("%02d",$sec)." \t ";
	  }
	elsif ($art==4)
	  {
	    $zeitmarke=sprintf("%04d",$year)."-".sprintf("%02d",$mon)."-".sprintf("%02d",$mday)." ".sprintf("%02d",$hour).":".sprintf("%02d",$min).":".sprintf("%02d",$sec)." \t ";
	  }
      }
    print $zeitmarke."down: ".$trafficreceived." ".$trafficreceivedeinheit." \t up: ".$traffictransmited." ".$traffictransmitedeinheit."\n";
    STDOUT->autoflush(1);
    if ($n==0) # endlos messen
      {
	$i=0;
      }
  }

